package ssw.mj.symtab;

/**
 * MicroJava Symbol Table Objects:
 * Every named object in a program is stored in an <code>Obj</code> node.
 * Every scope has a list of objects declared within it.
 */
public class Obj {
	public static final int 
		Con = 0, Var = 1, Type = 2, Meth = 3, Prog = 4;
	
	public int kind;     // Con, Var, Type, Meth, Prog
	public String name;
	public Struct type;
	public Obj next;     // next Obj node in this scope
	public int val;      // Con: value;
	public int adr;      // Var, Meth: offset
	public int level;    // Var: declaration level (0..global, 1..local)
	public int nPars;    // Meth: # of parameters
	public Obj locals;   // Meth: list of local variables
	                     // Prog: symbol table of program
	
	public Obj (int kind, String name, Struct type) {
		this.kind = kind; this.name = name; this.type = type;
	}

	/**
	 * String representation for debugging.
	 */
	@Override
	public String toString () {
		StringBuilder sb = new StringBuilder();
		dump(sb, "", false, false);
		return sb.toString();
	}

	/**
	 * String representation for JUnit test cases.
	 */
	public void dump(StringBuilder sb, String indent, boolean isField, boolean follow) {
		sb.append(indent);

		switch (kind) {
			case Con: 
				sb.append("Constant: "); 
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name).append(" = ");
				if (type == Tab.charType) {
					sb.append("'").append((char) val).append("'");
				} else {
					sb.append(val);
				}
				break;
			case Var: 
				if (isField) {
					sb.append("Field ");
				} else if (level == 0) {
					sb.append("Global Variable ");
				} else {
					sb.append("Local Variable ");
				}
				sb.append(adr).append(": ");
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name);
				break;
			case Type: 
				sb.append("Type ").append(name).append(": ");
				if (type != null) {
					type.dump(sb, indent + "  ", true);
				}
				break;
			case Meth: 
				sb.append("Method: "); 
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name).append("(").append(nPars).append(") at ").append(adr);
				break;
			case Prog: 
				sb.append("Program ").append(name).append(":");
				break;
		}

		if (follow && locals != null) {
			sb.append("\n");
			locals.dump(sb, indent + "  ", isField, follow);
		}
		if (follow && next != null) {
			sb.append("\n");
			next.dump(sb, indent, isField, follow);
		}
	}
}
